@@ -6,7 +6,7 @@
 # Copyright (C) 2006 Yann Hodique <hodique@lifl.fr>
 # Copyright (C) 2006 Lukas Lalinsky <lalinsky@gmail.com>
 # Copyright (C) 2006 Marien Zwart <marienz@gentoo.org>
-# Copyright (C) 2006, 2007 Panoramic Feedback <abentley@panoramicfeedback.com>
+# Copyright (C) 2006,2007 Panoramic Feedback <abentley@panoramicfeedback.com>
 # Copyright (C) 2006,2008,2009 Jelmer Vernooij <jelmer@samba.org>
 # Copyright (C) 2009,2010 Martin von Gagern <Martin.vGagern@gmx.net>
 # All rights reserved.
@@ -40,7 +40,7 @@ import re
 import fnmatch
 
 import trac
-from trac import versioncontrol, core, config, mimeview, wiki
+from trac import versioncontrol, core, config, mimeview, util, wiki
 from trac.web.chrome import Chrome
 from trac.util.html import html, Markup
 
@@ -86,6 +86,13 @@ class BzrConnector(core.Component):
         if hasattr(self.env, 'systeminfo'):
             bzr_version = '%i.%i.%i' % bzrlib.api.get_current_api_version(None)
             self.env.systeminfo.append(('Bazaar', bzr_version))
+            try:
+                import tracbzr as tracbzr_mod
+                tracbzr_version = util.get_pkginfo(tracbzr_mod).get('version')
+            except Exception, e:
+                self.log.warn(e)
+                tracbzr_version = 'unknown'
+            self.env.systeminfo.append(('TracBzr', tracbzr_version))
 
     def get_supported_types(self):
         """Support for `repository_type = bzr`"""
@@ -233,7 +240,7 @@ class BzrRepository(versioncontrol.Repository):
     """Present a bzr branch as a trac repository."""
 
     primary_branches = config.ListOption(
-        config_section, 'primary_branches', ',trunk', keep_empty = True, 
+        config_section, 'primary_branches', 'trunk', keep_empty = True, 
         doc = """Ordered list of primary branches.
 
         These will be listed first in the Branches macro. When viewing
@@ -243,7 +250,20 @@ class BzrRepository(versioncontrol.Repository):
         empty list element can be used to denote the branch at the
         root of the repository.
 
-        Defaults to ',trunk'.""")
+        Defaults to 'trunk'.""")
+
+    include_sideline_changes = config.BoolOption(
+        config_section, 'include_sideline_changes', True,
+        doc = """Include sideline changes in the list of changes.
+
+        This option controls whether sideline changes (i.e. changes
+        with dotted revision numbers only) are included in the list of
+        changes as reported by the timeline view. Note that there
+        might be other plugins using that information as well, so
+        there might be other components beside the timeline view that
+        get affected by this setting.
+
+        Defaults to True.""")
 
     def __init__(self, location, component):
         versioncontrol.Repository.__init__(self, location, None, component.log)
@@ -506,13 +526,14 @@ class BzrRepository(versioncontrol.Repository):
             else:
                 rhs = []
             rhss.append(rhs)
-        for (relpath, branch), rhs in zip(branches, rhss):
-            cache = self.get_branch_cache(branch)
-            while rhs:
-                revid = rhs.pop()
-                new_rhs = walk(cache, revid)
-                if new_rhs:
-                    rhs.extend(new_rhs)
+        if self.include_sideline_changes:
+            for (relpath, branch), rhs in zip(branches, rhss):
+                cache = self.get_branch_cache(branch)
+                while rhs:
+                    revid = rhs.pop()
+                    new_rhs = walk(cache, revid)
+                    if new_rhs:
+                        rhs.extend(new_rhs)
         inrange.sort()
         return [chgset for time, revid, chgset in inrange]
 
@@ -847,7 +868,9 @@ class UnversionedDirNode(BzrNode):
                        self.bzr_repo.string_rev(self.branch, NULL_REVISION),
                        versioncontrol.Changeset.ADD)
                 return
-        yield (self.path, BzrRepository._escape_revid(CURRENT_REVISION), 'add')
+        yield (self.path,
+               BzrRepository._escape_revid(CURRENT_REVISION),
+               versioncontrol.Changeset.ADD)
         return
 
 
@@ -879,7 +902,9 @@ class EmptyBranchNode(BzrNode):
         return None
 
     def get_history(self, limit=None):
-        return []
+        yield (self.path, 
+               self.bzr_repo.string_rev(self.branch, NULL_REVISION),
+               versioncontrol.Changeset.ADD)
 
 
 class BzrVersionedNode(BzrNode):
@@ -895,14 +920,16 @@ class BzrVersionedNode(BzrNode):
 
     def __init__(self, bzr_repo, branch, revisiontree, entry, path):
         """Initialize. path has to be a normalized path."""
-        rev_string = bzr_repo.string_rev(branch, entry.revision)
-        BzrNode.__init__(self, path, rev_string, self.kind)
+        # We must Iinitialize bzr specific stuff before we call
+        # get_content_revision().
         self.bzr_repo = bzr_repo
         self.log = bzr_repo.log
         self.repo = branch.repository
         self.branch = branch
         self.tree = revisiontree
         self.entry = entry
+        rev_string = bzr_repo.string_rev(branch, self.get_content_revision())
+        BzrNode.__init__(self, path, rev_string, self.kind)
         # XXX I am not sure if this makes any sense but it does make
         # the links in the changeset viewer work.
         self.created_rev = self.rev
@@ -1031,7 +1058,13 @@ class BzrVersionedNode(BzrNode):
                 current_tree = previous_tree
 
     def get_content_revision(self):
-        """For non-directory nodes, the content was last changed here."""
+        """Obtain the revision at which this node was last changed.
+        This should include changes to any children.
+        This will be called by BzrVersionedNode's constructor, so overriding
+        subclasses should ensure that this method can be called safely before
+        calling BzrVersionedNode.__init__().
+        """
+        # For non-directory nodes, the content was last changed here.
         return self.entry.revision
 
     def get_last_modified(self):
@@ -1046,13 +1079,11 @@ class BzrDirNode(BzrVersionedNode):
 
     def __init__(self, bzr_repo, branch, revisiontree, entry, path,
                  revcache=None):
-        BzrVersionedNode.__init__(self, bzr_repo, branch, revisiontree, entry, 
-                                  path)
         if revcache is None:
             revcache = _FileToRevisionCache(bzr_repo, branch, revisiontree, entry)
         self.revcache = revcache
-        self.rev = self.bzr_repo.string_rev(self.branch,
-                                            self.get_content_revision())
+        BzrVersionedNode.__init__(self, bzr_repo, branch, revisiontree, entry, 
+                                  path)
 
     def get_content_revision(self):
         """Determine the most recent change to the directory or its children."""
